/*
--             This file is part of the New World OS project
--                 Copyright (C) 2007-2008  QRW Software
--           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
--                      http://www.qrwsoftware.com
--                      http://nwos.sourceforge.com
--
--   This program is free software: you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation, either version 3 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program, in the file LICENSE.  If not, see 
--   <http://www.gnu.org/licenses/>.
--
--   You can also contact me via paper mail at:
--
--      QRW Software
--      P.O. Box 27511
--      Salt Lake City, UT 84127-0511, USA.
--
--
-- $Log: fix_0030_compressed_zero.c,v $
-- Revision 1.1  2009/07/18 04:10:20  jsedwards
-- Program to repair file with zeros in upper bytes of reference in blocks.
--
-- Revision 1.3  2008/09/01 00:23:02  jsedwards
-- Fix year in copyright.  NO code changes.
--
-- Revision 1.2  2008/01/02 16:43:52  jsedwards
-- New method: read the whole file into memory and rearrange the blocks.
--
-- Revision 1.1  2008/01/02 14:14:35  jsedwards
-- First attempt at a program to repair a broken .dif file, DOES NOT WORK!
--
*/

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>    /* define sleep() */

#include "../disk_io.h"


uint32 convert_4_uint8_to_uint32(uint8 byte[4])
{
  return ((uint32)byte[0] << 24) | ((uint32)byte[1] << 16) | ((uint32)byte[2] << 8) | (uint32)byte[3];
}


int main(int argc, char* argv[])
{
    FILE* fp1;
    size_t read1;
    size_t write2;
    uint8 block[FILE_BLOCK_SIZE];
    size_t num_blocks = 0;


    if (argc != 2)
    {
	fprintf(stderr, "usage: %s in_out_file\n", argv[0]);
	exit(1);
    }


    /* Open the file and check it */

    fp1 = fopen(argv[1], "r+");
    if (fp1 == NULL)
    {
	perror(argv[1]);
	exit(1);
    }


    read1 = fread(block, 1, FILE_BLOCK_SIZE, fp1);

    while (!feof(fp1) && read1 == FILE_BLOCK_SIZE)
    {
	if (num_blocks < 1)
	{
	    if (block[0] != 'N' || block[1] != 'W' || block[2]  != 'O' || block[3]  != 'S' ||
		block[4] != '0' || block[5] != '0' || block[6]  != '3' || block[7]  != '0' ||
		block[8] != 'c' || block[9] != 'm' || block[10] != 'p' || block[11] != 'r')
	    {	
		fprintf(stderr, "Not a NWOS 0030 compressed file\n");
		exit(1);
	    }
	}
	else
	{
	    if (block[0] != 0 || block[1] != 0 || block[2] != 0 || block[3]  != 0)
	    {
		fprintf(stderr, "Block: %d upper bytes non-zero: %02x%02x%02x%02x\n", num_blocks, block[0], block[0], block[2], block[3]);
		exit(1);
	    }

	    block[0] = 0xff;
	    block[1] = 0xff;
	    block[2] = 0xff;
	    block[3] = 0xfe;

	    if (fseek(fp1, -FILE_BLOCK_SIZE, SEEK_CUR) != 0)
	      {
		perror(argv[1]);
		exit(1);
	      }

	    write2 = fwrite(block, 1, FILE_BLOCK_SIZE, fp1);

	    if (write2 != FILE_BLOCK_SIZE)
	    {
		perror(argv[1]);
		exit(1);
	    }
	}

	num_blocks++;

	read1 = fread(block, 1, FILE_BLOCK_SIZE, fp1);
    }

    if (!feof(fp1) || ferror(fp1))
    {
	perror(argv[1]);
	exit(1);
    }

    printf("file size - blocks: %d\n", num_blocks);

    if (fclose(fp1) != 0)
    {
	perror(argv[1]);
	exit(1);
    }

    return 0;
}
